/* 
 * Copyright (c) 2007, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.samples.pendingagreement;

import java.util.Map;

import javax.security.auth.login.LoginContext;
import javax.security.auth.login.LoginException;
import javax.xml.namespace.QName;

import org.apache.log4j.Logger;
import org.ogf.graap.wsag.api.Agreement;
import org.ogf.graap.wsag.api.AgreementOffer;
import org.ogf.graap.wsag.api.exceptions.AgreementFactoryException;
import org.ogf.graap.wsag.api.security.KeystoreLoginContext;
import org.ogf.graap.wsag.api.security.KeystoreProperties;
import org.ogf.graap.wsag.api.security.SecurityProperties;
import org.ogf.graap.wsag.api.types.AgreementOfferType;
import org.ogf.graap.wsag.client.api.AgreementClient;
import org.ogf.graap.wsag.client.api.AgreementFactoryClient;
import org.ogf.graap.wsag.server.actions.AbstractCreateAgreementAction;
import org.ogf.graap.wsag.server.actions.ActionInitializationException;
import org.ogf.graap.wsag.server.api.AgreementAcceptanceFactory;
import org.ogf.graap.wsag.server.api.AgreementAcceptanceListener;
import org.ogf.schemas.graap.wsAgreement.AgreementStateDefinition;
import org.ogf.schemas.graap.wsAgreement.AgreementTemplateType;
import org.ogf.schemas.graap.wsAgreement.ServiceDescriptionTermType;
import org.w3.x2005.x08.addressing.EndpointReferenceDocument;
import org.w3.x2005.x08.addressing.EndpointReferenceType;

/**
 * CreatePendingAgreementAction
 * 
 * This class shows an example for a long lasting agreement creation process.
 * 
 * @author Oliver Waeldrich
 * 
 */
public class CreatePendingAgreementWithNotificationAction extends AbstractCreateAgreementAction
{

    private static final Logger LOG = Logger.getLogger( CreatePendingAgreementWithNotificationAction.class );

    /**
     * Example for a long lasting agreement creation process.
     * 
     * {@inheritDoc}
     */
    @Override
    public Agreement createAgreement( AgreementOffer offer, Map<String, Object> invocationCtx )
        throws AgreementFactoryException
    {
        AgreementAcceptanceFactory acceptance =
            (AgreementAcceptanceFactory) invocationCtx.get( AgreementAcceptanceFactory.AGREEMENT_ACCEPTANCE_FACTORY );

        if ( acceptance == null )
        {
            throw new AgreementFactoryException( "Acceptance not supported." );
        }
        else
        {
            //
            // create an acceptance instance and register the acceptance handler
            //
            final PendingAgreement agreement = new PendingAgreement( offer );

            AgreementAcceptanceListener acceptanceListener = new AgreementAcceptanceListener()
            {

                @Override
                public void reject()
                {
                    try
                    {
                        agreement.getState().setState( AgreementStateDefinition.REJECTED );
                    }
                    catch ( Exception e )
                    {
                        LOG.error( "Error while setting agreement state." );
                    }
                }

                @Override
                public void accept()
                {
                    try
                    {
                        agreement.getState().setState( AgreementStateDefinition.COMPLETE );
                    }
                    catch ( Exception e )
                    {
                        LOG.error( "Error while setting agreement state." );
                    }
                }
            };

            EndpointReferenceType acceptanceEPR =
                acceptance.registerAgreementAccetanceListener( acceptanceListener );

            //
            // now create an pending agreement with the factory specified
            // in the agreement offer
            //
            final QName eprQName = EndpointReferenceDocument.type.getDocumentElementName();
            ServiceDescriptionTermType sdt = offer.getTerms().getAll().getServiceDescriptionTermArray( 0 );
            EndpointReferenceType epr = (EndpointReferenceType) sdt.selectChildren( eprQName )[0];

            try
            {
                SecurityProperties securityProperties = new SecurityProperties( getLoginContext() );

                AgreementFactoryClient factory =
                    AgreementFactoryClient.FACTORY.newInstance( epr, securityProperties.getLoginContext() );

                AgreementTemplateType template = factory.getTemplate( "SAMPLE4-PENDING-AGREEMENT", "1" );
                AgreementClient agreementClient =
                    factory.createPendingAgreement( new AgreementOfferType( template ), acceptanceEPR );

                if ( LOG.isInfoEnabled() )
                {
                    LOG.info( "Created agreement client: "
                        + agreementClient.getRemoteClient().getRemoteReference().xmlText() );
                }
            }
            catch ( Exception e )
            {
                LOG.error( "Failed to create the pending agreement. Reason: " + e.getMessage() );

                if ( LOG.isDebugEnabled() )
                {
                    LOG.debug( e );
                }

                throw new AgreementFactoryException( e );
            }

            //
            // Now, since the work is done and we return the
            // pending, subcontracted agreement...
            //
            return agreement;

        }
    }

    private LoginContext getLoginContext() throws LoginException
    {
        //
        // appropriate implementation has to be provided by an implementation
        //

        /*
         * create example keystore login context
         */
        LOG.info( "Create Keystore-LoginContext" );

        // START SNIPPET: CreateLoginContext
        KeystoreProperties properties = new KeystoreProperties();
        properties.setKeyStoreAlias( "wsag4j-server" );
        properties.setPrivateKeyPassword( "server@wsag4j" );

        properties.setKeyStoreType( "JKS" );
        properties.setKeystoreFilename( "/wsag4j-server-keystore.jks" );
        properties.setKeystorePassword( "server@wsag4j" );

        properties.setTruststoreType( "JKS" );
        properties.setTruststoreFilename( "/wsag4j-server-keystore.jks" );
        properties.setTruststorePassword( "server@wsag4j" );

        LoginContext loginContext = new KeystoreLoginContext( properties );
        loginContext.login();

        return loginContext;
    }

    /**
     * Initializes the action.
     * 
     * {@inheritDoc}
     */
    @Override
    public void initialize() throws ActionInitializationException
    {
        // nothing to do here
    }

}
